var express = require('express')
  , expressNamespace = require('express-namespace')
  , fs = require('fs')
  , vm = require('vm')
  , stat = fs.statSync
  , ls = fs.readdirSync
  , path = require('path')
  , util = require('util')
  , inspect = util.inspect
  , join = path.join

  , fructose = require('fructose')
  

var exedra = function(Variables) {
  var variables = fructose.merge({ }, global, Variables, { require: require })
    , cwd = process.cwd()
    , app = variables.app
    , fn = {}
  
  fn.functions = function(ctx){
    var dir = app.set('path.functions')
    if(!dir)
      throw new Error('App "path.functions" setting is undefined')
      
    var loadRecursive = function(dir, link){
      var result = link || {}

      ls(dir).forEach(function(name) {
        if(fructose.notTempOrHidden(name)) {
          var path = join(dir, name)
            , key = path.replace(dir, '').replace(/^\//, '').replace(/\.js$/i, '')

          if(!result[key])
            result[key] = {}
          
          if(stat(path).isDirectory()) {
            loadRecursive(path, result[key])
          } else {
            var module = require(path)
            if(module instanceof Function) {
              var context = vm.createContext(ctx ? fructose.merge({}, variables, ctx) : variables)
                , runtime = vm.runInContext('(' + module.toString() + ')()', context)
                
              result[key] = fructose.merge(result[key], runtime)
            }
          }
        }
      })

      return result
    }
    
    variables.fn = loadRecursive(dir)
    
    return this
  }


  /* ->
   * End of stack
   */
  fn.routes = function(ctx){
    var dir = app.set('path.routes')
    if(!dir)
      throw new Error('App "path.routes" setting is undefined')
    
    var namespaces = {}
      
    fructose.lsRecursive(dir).forEach(function(route) {
      route = route.replace(dir, '').replace(/\.js$/i, '')
      
      var path = join(dir, route)
      
      route = route.replace(/index$/i, '').replace(/\/index/, '/')
      
      if(!namespaces[route])
        namespaces[route] = []
      
      namespaces[route].push({
        route: route,
        init: require(path),
        path: path
      })
      
    })
    
    namespaces = fructose.toArray(namespaces)
    namespaces.forEach(function(routes){
      app.namespace(routes[0].route, function(){
        var self = this
          , context = vm.createContext(fructose.merge({ namespace: self }, variables, ctx || {}))

        routes.forEach(function(route){
          if(typeof route.init == 'function')
            vm.runInContext('(' + route.init.toString() + ').apply(namespace)', context)
          else
            console.error('  \033[31;2m%s\033[0m exports not function', route.path)
        })
        
      })
    })
    
    return this
  }
  
  return fn;

}
module.exports = exedra
